@@ -17,13 +17,6 @@ try:
 except:
     SKIMAGE_ENABLED = False
 
-try:
-    from omnipose.utils import sinebow
-    import ncolor
-    OMNI_INSTALLED = True
-except:
-    OMNI_INSTALLED = False
-
 # modified to use sinebow color
 def dx_to_circ(dP,transparency=False,mask=None):
     """ dP is 2 x Y x X => 'optic' flow representation 
@@ -61,7 +54,7 @@ def dx_to_circ(dP,transparency=False,mask=None):
     im = (np.clip(im, 0, 1) * 255).astype(np.uint8)
     return im
 
-def show_segmentation(fig, img, maski, flowi, channels=[0,0], file_name=None, omni=False, seg_norm=False, bg_color=None):
+def show_segmentation(fig, img, maski, flowi, channels=[0,0], file_name=None):
     """ plot segmentation results (like on website)
     
     Can save each panel of figure with file_name option. Use channels option if
@@ -88,15 +81,9 @@ def show_segmentation(fig, img, maski, flowi, channels=[0,0], file_name=None, om
     file_name: str (optional, default None)
         file name of image, if file_name is not None, figure panels are saved
         
-    omni: bool (optional, default False)
-        use omni version of normalize99, image_to_rgb
-        
     seg_norm: bool (optional, default False)
         improve cell visibility under labels
         
-    bg_color: float (Optional, default none)
-        background color to draw behind flow (visible if flow transparency is on)
-        
 
     """
     if not MATPLOTLIB_ENABLED:
@@ -107,7 +94,7 @@ def show_segmentation(fig, img, maski, flowi, channels=[0,0], file_name=None, om
     if img0.shape[0] < 4:
         img0 = np.transpose(img0, (1,2,0))
     if img0.shape[-1] < 3 or img0.ndim < 3:
-        img0 = image_to_rgb(img0, channels=channels, omni=omni)
+        img0 = image_to_rgb(img0, channels=channels)
     else:
         if img0.max()<=50.0:
             img0 = np.uint8(np.clip(img0*255, 0, 1))
@@ -117,24 +104,7 @@ def show_segmentation(fig, img, maski, flowi, channels=[0,0], file_name=None, om
 
     outlines = utils.masks_to_outlines(maski)
 
-    # Image normalization to improve cell visibility under labels
-    if seg_norm:
-        fg = 1/9
-        p = np.clip(transforms.normalize99(img0,omni=omni), 0, 1)
-        img1 = p**(np.log(fg)/np.log(np.mean(p[maski>0])))
-    else:
-        img1 = img0
-    
-    # the mask_overlay function changes colors (preserves only hue I think). The label2rgb function from
-    # skimage.color works really well. 
-    if omni and SKIMAGE_ENABLED and OMNI_INSTALLED:
-        c = sinebow(5)
-        colors = np.array(list(c.values()))[1:] 
-        overlay = color.label2rgb(ncolor.label(maski),img1,colors,bg_label=0,alpha=1/3)
-        overlay = np.uint8(np.clip(overlay, 0, 1)*255)
-        overlay[maski==0] = img1[maski==0] #restore original level to background regions
-    else:
-        overlay = mask_overlay(img0, maski)
+    overlay = mask_overlay(img0, maski)
 
     ax = fig.add_subplot(1,4,2)
     outX, outY = np.nonzero(outlines)
@@ -151,9 +121,6 @@ def show_segmentation(fig, img, maski, flowi, channels=[0,0], file_name=None, om
     ax.axis('off')
 
     ax = fig.add_subplot(1,4,4)
-    if bg_color is not None:
-        ax.imshow(np.ones_like(flowi)*bg_color)
-    
     ax.imshow(flowi)
     ax.set_title('predicted cell pose')
     ax.axis('off')
@@ -202,7 +169,7 @@ def mask_rgb(masks, colors=None):
     RGB = (utils.hsv_to_rgb(HSV) * 255).astype(np.uint8)
     return RGB
 
-def mask_overlay(img, masks, colors=None, omni=False):
+def mask_overlay(img, masks, colors=None):
     """ overlay masks on image (set image to grayscale)
 
     Parameters
@@ -247,7 +214,7 @@ def mask_overlay(img, masks, colors=None, omni=False):
     RGB = (utils.hsv_to_rgb(HSV) * 255).astype(np.uint8)
     return RGB
 
-def image_to_rgb(img0, channels=[0,0], omni=False):
+def image_to_rgb(img0, channels=[0,0]):
     """ image is 2 x Ly x Lx or Ly x Lx x 2 - change to RGB Ly x Lx x 3 """
     img = img0.copy()
     img = img.astype(np.float32)
@@ -259,7 +226,7 @@ def image_to_rgb(img0, channels=[0,0], omni=False):
         img = img.mean(axis=-1)[:,:,np.newaxis]
     for i in range(img.shape[-1]):
         if np.ptp(img[:,:,i])>0:
-            img[:,:,i] = np.clip(transforms.normalize99(img[:,:,i],omni=omni), 0, 1)
+            img[:,:,i] = np.clip(transforms.normalize99(img[:,:,i]), 0, 1)
             img[:,:,i] = np.clip(img[:,:,i], 0, 1)
     img *= 255
     img = np.uint8(img)
